#!/bin/bash
#
# (c) astx (at) aws-it.at, 2018 - 2019
# and (c) others - see DIYaudio thread https://www.diyaudio.com/forums/software-tools/331656-installing-using-ngspice-opensource-simulator.html
#
# $Id: step_vbic_models.sh 1607 2019-02-23 14:43:47Z sta $
#
echo ""; echo ""
echo "#-----------------------------------------------------------------------"
echo "NOTE: For successful run you need at least ngspice 30 + pre-master fixes"
echo "After simulation run you can further explore the result data"
echo "Use 'quit' to exit simulation"
echo ""
echo "#-----------------------------------------------------------------------"
echo "If you want to run opamp stepping simulation now enter 'y' <ENTER>"
echo "#-----------------------------------------------------------------------"

fork_ngspice_process () {
	local ngspice
	mynet=$1
	bjt=$2
	echo "*** fork ngspice run for device ${mynet}.${bjt}.inc"
	ngspice -i ${mynet}.${bjt}.inc curve_tracer.ngspice > tmp_curve_log_${bjt}.data 2>&1
	echo "*** finished ngspice run for device $bjt"
}

print_netlist () {
	local bjt=$1
	local typ=$2
	local netlist="
*
.control
* --------- BJT specific values -------
	* ngspice-30 BUG: need a non numeric character and not +/- in front of the bjt name, otherwise the bjt name will be converted to a number
	set mybjt=\"_$bjt\"
	set ibase1=( 2u 4u 6u 8u 10u 12u 14u 16u )
	set sweep1end=5
	set ibase2=( 0.2u 0.4u 0.6u 0.8u 1u 1.2u 1.4u 1.6u )
	set sweep2end=100
* --------- BJT specific values -------
.endc

Q1 net1 net2 0 0 $bjt"

	case "$typ" in
		npn)
			echo -en "${netlist}\n"
			echo -en "Vce net1 0 DC=0 AC=0\n"
			echo -en "I1 0 net2 1u\n.end\n"
			;;
		pnp)
			echo -en "${netlist}\n"
			echo -en "Vce 0 net1 DC=0 AC=0\n"
			echo -en "I1 net2 0 1u\n.end\n"
			;;
	esac
}

IFS="
"

read a
if [ "$a" == "y" ]; then

	# change this to your preferred editor. vi nano joe ...
	export SPICE_EDITOR="joe"

	numcpu=4
	if [ -f /proc/cpuinfo ]; then
		numcpu=$(cat /proc/cpuinfo |grep processor|wc -l);
		echo "found $numcpu processors"
	fi

	# temporary net list files - need uniqe name for parallel processing!
	mynet="tmp_curve_trace"
	rm curve_*_trace[1-2].{data,plt,png}
	rm tmp_curve_trace.*.inc
	rm tmp_curve_log_*.data

	count=1
	for line in $(cat kean9_vbic.lib|grep -i '^.model'|grep -i -v 'AKO:'|sed -r -e 's/\s+/ /g' |cut -d' ' -f2-3)
	do
		bjt=$(echo $line|cut -d' ' -f1)
		typ=$(echo $line|cut -d' ' -f2|tr 'A-Z' 'a-z')

		if [ "$typ" != "npn" ] && [ "$typ" != "pnp" ]; then
			echo "*** skip unknown model type: $line"
			continue
		fi
		# e.g. add/set some circuit params or options be4 simulation run
		print_netlist ${bjt} ${typ} > ${mynet}.${bjt}.inc

		# fork process into background
		fork_ngspice_process ${mynet} ${bjt} &

		# wait with loop continuation if too much processes are running
		while [ $(ps axf|grep "${mynet}" |grep -v grep|wc -l) -gt ${numcpu} ]
		do
			# echo -en '.'
			echo 'WAIT_FORKING'
			usleep 50000
		done
		echo "continue with loop"

		count=$(($count+1))
	done
fi
